clc; clear all;

%Load Data to the Code
ConcTable = [   -1 0 1;
     -1 0 1;
     -1 0 1;
     -1 0 1;
     -1 0 1;
     -1 0 1;
     -1 0 1;
     -1 0 1;        
                ];
    
Row = 59; Col = 8;
filename = 'WisDM_Green_Data';
xlrange = 'b2:j60'; data = xlsread(filename,1,xlrange);
in_data = data(1:Row,1:Col); out_data = data(1:Row,Col+1);
X_o = in_data; Y_o = out_data;
SHIFT_VAL = abs(min(Y_o)) + 0.01;
Y_o = Y_o + SHIFT_VAL;
Y_o = Y_o.^2;

%WisDM Green Analysis
result = stepwiselm(X_o,Y_o,'quadratic','ResponseVar','Inhibition','PredictorVars',{'D1','D2','D3','D4','D5','D6','D7','D8'}); %, 'PEnter', 0, 'PRemove', 0.05);
beta = result.Coefficients.Estimate; disp(result); Y_o_rg = result.Fitted;
R = corr(Y_o,Y_o_rg); str = ['Fitting Correlation is: ', num2str(R)]; disp(str);

NOC = size(ConcTable,2);
    [D1,D2,D3,D4,D5,D6,D7,D8]=ndgrid(1:1:NOC);
    X_All = [D1(:) D2(:) D3(:) D4(:) D5(:) D6(:) D7(:) D8(:)];
    for i=1:size(X_All,1)
    for j=1:size(X_All,2)                                
        X_All(i,j) = ConcTable(j,X_All(i,j)); 
    end
    end

%Plot Figures    
figure('Name','Result')
subplot(2,2,1); plotResiduals(result,'Fitted');
subplot(2,2,2); plotDiagnostics(result,'cookd');
subplot(2,2,3); plotResiduals(result,'probability')
subplot(2,2,4); plotResiduals(result,'histogram') 
plotSlice(result); figure('Name', 'Interactions')
plotInteraction(result,'D1','D3','predictions'); 

%Calculate All Possible Combinations
T=result.Formula.Terms; T=T(1:end,1:Col);   
X_All_rg = x2fx(X_All,T); Y_All_rg=X_All_rg*beta;

Y_All_rg = sqrt(Y_All_rg)- SHIFT_VAL;

[Opt_Y, ind] = max(Y_All_rg); Opt_Comb = X_All(ind,:);
str = ['Optimal Output of Y is ',num2str(Opt_Y)]; disp(str);
str1 = 'Optimal Combination of Y is ';
str = [str1,num2str(Opt_Comb)]; disp(str);
ypred = predict(result,Opt_Comb);
FULL = [X_All Y_All_rg];

%Produce Subset of Compounds (e.g. 4 or less)
idx = X_All ~= -1; Numb_Drug = sum(idx,2);
FULL_Numb = [FULL Numb_Drug];
FULL_Numb = sortrows(FULL_Numb, 9, 'descend');
Rank = [1:height(X_All)]';
FULL_TOTAL = [FULL_Numb Rank];
SUBSET = sortrows(FULL_TOTAL, 10);
SUBSET_FINAL = SUBSET(:,10)  > 4;  SUBSET(SUBSET_FINAL,:) = [];
SUBSET = sortrows(SUBSET, 11);

% %Save Full Data and Subset
% xlswrite('FULL_DATA.xlsx', FULL_TOTAL);
% xlswrite('SUBSET_DATA.xlsx', SUBSET);